@@ -1,13 +1,13 @@
+module Alias = Alias0
+module Findlib = Findlib
 module Main = Main
 module Context = Context
 module Super_context = Super_context
 module Compilation_context = Compilation_context
-module Findlib = Findlib
 module Colors = Colors
-module Profile = Profile
 module Workspace = Workspace
 module Dune_package = Dune_package
-module Dep_conf = Dep_conf
+module Alias_rec = Alias_rec
 module Dir_contents = Dir_contents
 module Expander = Expander
 module Lib = Lib
@@ -31,7 +31,6 @@ module Opam_create = Opam_create
 module Link_mode = Link_mode
 module Utop = Utop
 module Setup = Setup
-module Meta = Meta
 module Toplevel = Toplevel
 module Top_module = Top_module
 module Global = Global
@@ -40,25 +39,32 @@ module Resolve = Resolve
 module Ocamldep = Ocamldep
 module Dep_rules = Dep_rules
 module Dep_graph = Dep_graph
+module Lib_config = Lib_config
 module Preprocess = Preprocess
+module Preprocessing = Preprocessing
+module Coq_mode = Coq_mode
 module Coq_rules = Coq_rules
 module Coq_module = Coq_module
 module Coq_sources = Coq_sources
-module Coq_stanza = Coq_stanza
+module Coq_lib_name = Coq_lib_name
 module Coq_lib = Coq_lib
 module Command = Command
-module Install = Install
-module Lib_name = Lib_name
-module Diff = Dune_lang.Action.Diff
+module Clflags = Clflags
+module Dune_project = Dune_project
+module Source_tree = Source_tree
+module Sub_dirs = Sub_dirs
+module Package = Package
+module Dialect = Dialect
+module Private_context = Private_context
 
 module Install_rules = struct
   let install_file = Install_rules.install_file
+  let stanzas_to_entries = Install_rules.stanzas_to_entries
 end
 
-(* Only for tests *)
-module Scheme = Scheme
-module Lib_config = Lib_config
-module Dynlink_supported = Dynlink_supported
-module Lib_dep = Lib_dep
-module Ocamlobjinfo = Ocamlobjinfo
-module Action_unexpanded = Action_unexpanded
+module For_tests = struct
+  module Dynlink_supported = Dynlink_supported
+  module Ocamlobjinfo = Ocamlobjinfo
+  module Action_unexpanded = Action_unexpanded
+  module Jsoo_rules = Jsoo_rules
+end
