@@ -9,7 +9,7 @@ end
 
 module Implementation : sig
   type t =
-    { existing_virtual_modules : Module_name.Set.t
+    { existing_virtual_modules : Module_name.Path.Set.t
     ; allow_new_public_modules : bool
     }
 end
@@ -19,13 +19,12 @@ type kind =
   | Implementation of Implementation.t
   | Exe_or_normal_lib
 
-val eval :
-     modules:Module.Source.t Module_name.Map.t
+val eval
+  :  modules:Module.Source.t Module_trie.t
   -> stanza_loc:Loc.t
-  -> modules_field:Ordered_set_lang.t
-  -> modules_without_implementation:Ordered_set_lang.t
-  -> root_module:('a * Module_name.t) option
   -> private_modules:Ordered_set_lang.t
   -> kind:kind
   -> src_dir:Path.Build.t
-  -> Module.t Module_name.Map.t
+  -> version:Dune_lang.Syntax.Version.t
+  -> Stanza_common.Modules_settings.t
+  -> ((Loc.t * Module.Source.t) Module_trie.t * Module.t Module_trie.t) Memo.t
