@@ -1,5 +1,4 @@
-open! Dune_engine
-open Stdune
+open Import
 
 (** The type of all kind of sub-system information. This type is what we get
     just after parsing a [dune] file. *)
@@ -11,7 +10,6 @@ type sub_system = t = ..
     sub-system. *)
 module type S = sig
   type t
-
   type sub_system += T of t
 
   (** Name of the sub-system *)
@@ -31,14 +29,11 @@ module type S = sig
   val encode : t -> Dune_lang.Syntax.Version.t * Dune_lang.t list
 end
 
-module Register (M : S) : sig end
+module Register (_ : S) : sig end
 
-val record_parser :
-     unit
-  -> ( sub_system Sub_system_name.Map.t
-     , Dune_lang.Decoder.fields )
-     Dune_lang.Decoder.parser
+val record_parser
+  :  unit
+  -> (sub_system Sub_system_name.Map.t, Dune_lang.Decoder.fields) Dune_lang.Decoder.parser
 
 val get : Sub_system_name.t -> (module S)
-
 val equal : t -> t -> bool
